==> libphidget22java-1.0.0.20171026.tar.gz: libphidget22java-1.0.0.20171026/src/jni/com_phidget22_GPS.c <==
/*
 * This file is part of libphidget22java
 *
 * Copyright 2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see
 * <http://www.gnu.org/licenses/>
 */

#include "phidget_jni.h"
#include "com_phidget22_GPS.h"
#include "com_phidget22_GPSBase.h"

extern jfieldID phidget_handle_fid;

static jclass calendar_class;
static jmethodID calendar_getInstance;
static jmethodID calendar_set;

static jclass timeZone_class; //class
static jmethodID timeZone_getTimeZone;

void com_phidget22_GPS_OnLoad(JNIEnv *env) {
	if (!(calendar_class = (*env)->FindClass(env, "java/util/Calendar")))
		JNI_ABORT_STDERR("Couldn't FindClass java.util.Calendar");
	if (!(calendar_class = (jclass)(*env)->NewGlobalRef(env, calendar_class)))
		JNI_ABORT_STDERR("Couldn't create global ref calendar_class");
	if (!(calendar_getInstance = (*env)->GetStaticMethodID(env, calendar_class, "getInstance", "(Ljava/util/TimeZone;)Ljava/util/Calendar;")))
		JNI_ABORT_STDERR("Couldn't get method ID getInstance");
	if (!(calendar_set = (*env)->GetMethodID(env, calendar_class, "set", "(II)V")))
		JNI_ABORT_STDERR("Couldn't get method ID set");

	if (!(timeZone_class = (*env)->FindClass(env, "java/util/TimeZone")))
		JNI_ABORT_STDERR("Couldn't FindClass java.util.TimeZone");
	if (!(timeZone_class = (jclass)(*env)->NewGlobalRef(env, timeZone_class)))
		JNI_ABORT_STDERR("Couldn't create global ref java.util.TimeZone");
	if (!(timeZone_getTimeZone = (*env)->GetStaticMethodID(env, timeZone_class, "getTimeZone", "(Ljava/lang/String;)Ljava/util/TimeZone;")))
		JNI_ABORT_STDERR("Couldn't get method ID  from getTimeZone");
}

JNIEXPORT jobject JNICALL
Java_com_phidget22_GPSBase_getDateAndTime(JNIEnv *env, jobject obj) {
	PhidgetGPSHandle h = (PhidgetGPSHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	int error;
	PhidgetGPS_Time cGPSTime;
	PhidgetGPS_Date cGPSDate;
	jobject GPSTimeAndDate, GPSTimeZone;

	if ((error = PhidgetGPS_getTime(h, &cGPSTime))) {
		PH_THROW(error);
		return NULL;
	}

	if ((error = PhidgetGPS_getDate(h, &cGPSDate))) {
		PH_THROW(error);
		return NULL;
	}

	//create a Java abstract TimeZone object with a time zone of UTC 
	if (!(GPSTimeZone=(*env)->CallStaticObjectMethod(env, timeZone_class, timeZone_getTimeZone,
		(*env)->NewStringUTF(env, "GMT")))) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	//calls the Java static method Calendar.getInstance object said TimeZone
	if (!(GPSTimeAndDate = (*env)->CallStaticObjectMethod(env, calendar_class, calendar_getInstance, GPSTimeZone))) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}
	(*env)->DeleteLocalRef(env, GPSTimeZone);

	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 1, cGPSDate.tm_year);  //year
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 2, (cGPSDate.tm_mon - 1)); //month(0-11)  
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 5, cGPSDate.tm_mday);  //day(1-31)

	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 11, (jint)cGPSTime.tm_hour);  //hour(0-23)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 12, cGPSTime.tm_min); //minute(0-59)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 13, cGPSTime.tm_sec);  //second(0-59)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 14, cGPSTime.tm_ms);  //milliseconds

	return GPSTimeAndDate;
}

==> libphidget22java-1.0.0.20180327.tar.gz: libphidget22java-1.0.0.20180327/src/jni/com_phidget22_GPS.c <==
/*
 * This file is part of libphidget22java
 *
 * Copyright 2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see
 * <http://www.gnu.org/licenses/>
 */

#include "phidget_jni.h"
#include "com_phidget22_GPS.h"
#include "com_phidget22_GPSBase.h"

extern jfieldID phidget_handle_fid;

static jclass calendar_class;
static jmethodID calendar_getInstance;
static jmethodID calendar_set;

static jclass timeZone_class; //class
static jmethodID timeZone_getTimeZone;

void com_phidget22_GPS_OnLoad(JNIEnv *env) {
	if (!(calendar_class = (*env)->FindClass(env, "java/util/Calendar")))
		JNI_ABORT_STDERR("Couldn't FindClass java.util.Calendar");
	if (!(calendar_class = (jclass)(*env)->NewGlobalRef(env, calendar_class)))
		JNI_ABORT_STDERR("Couldn't create global ref calendar_class");
	if (!(calendar_getInstance = (*env)->GetStaticMethodID(env, calendar_class, "getInstance", "(Ljava/util/TimeZone;)Ljava/util/Calendar;")))
		JNI_ABORT_STDERR("Couldn't get method ID getInstance");
	if (!(calendar_set = (*env)->GetMethodID(env, calendar_class, "set", "(II)V")))
		JNI_ABORT_STDERR("Couldn't get method ID set");

	if (!(timeZone_class = (*env)->FindClass(env, "java/util/TimeZone")))
		JNI_ABORT_STDERR("Couldn't FindClass java.util.TimeZone");
	if (!(timeZone_class = (jclass)(*env)->NewGlobalRef(env, timeZone_class)))
		JNI_ABORT_STDERR("Couldn't create global ref java.util.TimeZone");
	if (!(timeZone_getTimeZone = (*env)->GetStaticMethodID(env, timeZone_class, "getTimeZone", "(Ljava/lang/String;)Ljava/util/TimeZone;")))
		JNI_ABORT_STDERR("Couldn't get method ID  from getTimeZone");
}

JNIEXPORT jobject JNICALL
Java_com_phidget22_GPSBase_getDateAndTime(JNIEnv *env, jobject obj) {
	PhidgetGPSHandle h = (PhidgetGPSHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	int error;
	PhidgetGPS_Time cGPSTime;
	PhidgetGPS_Date cGPSDate;
	jobject GPSTimeAndDate, GPSTimeZone;
	jstring gmt;

	if ((error = PhidgetGPS_getTime(h, &cGPSTime))) {
		PH_THROW(error);
		return NULL;
	}

	if ((error = PhidgetGPS_getDate(h, &cGPSDate))) {
		PH_THROW(error);
		return NULL;
	}

	//create a Java abstract TimeZone object with a time zone of UTC
	gmt = (*env)->NewStringUTF(env, "GMT");
	if (!(GPSTimeZone=(*env)->CallStaticObjectMethod(env, timeZone_class, timeZone_getTimeZone, gmt))) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	//calls the Java static method Calendar.getInstance object said TimeZone
	if (!(GPSTimeAndDate = (*env)->CallStaticObjectMethod(env, calendar_class, calendar_getInstance, GPSTimeZone))) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	(*env)->DeleteLocalRef(env, GPSTimeZone);
	(*env)->DeleteLocalRef(env, gmt);

	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 1, cGPSDate.tm_year);  //year
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 2, (cGPSDate.tm_mon - 1)); //month(0-11)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 5, cGPSDate.tm_mday);  //day(1-31)

	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 11, (jint)cGPSTime.tm_hour);  //hour(0-23)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 12, cGPSTime.tm_min); //minute(0-59)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 13, cGPSTime.tm_sec);  //second(0-59)
	(*env)->CallVoidMethod(env, GPSTimeAndDate, calendar_set, 14, cGPSTime.tm_ms);  //milliseconds

	return GPSTimeAndDate;
}
